@@ -120,14 +120,19 @@ void learn_bias(BundleFactory& bundle_factory, BiasLearner& bl, bool progress_ba
 		if (progress_bar)
 			p_bar.update(bundle_label_buf, bundle.raw_mass());
 		
-		if (bundle.non_redundant_hits().size()==0 || bundle.ref_scaffolds().size() != 1)
+		if (bundle.non_redundant_hits().size()==0)// || bundle.ref_scaffolds().size() != 1)
 		{
 			delete bundle_ptr;
 			continue;
 		}
 
-		bl.preProcessTranscript(*(bundle.ref_scaffolds()[0]));
-			
+		//bl.preProcessTranscript(*(bundle.ref_scaffolds()[0]));
+		
+        for (size_t j = 0; j < bundle.ref_scaffolds().size(); ++j)
+        {
+            bl.preProcessTranscript(*(bundle.ref_scaffolds()[j]));
+        }
+        
 		delete bundle_ptr;
 	}
 	
@@ -136,8 +141,8 @@ void learn_bias(BundleFactory& bundle_factory, BiasLearner& bl, bool progress_ba
 		
 	bl.normalizeParameters();
     
-    if (output_bias_params)
-        bl.output();
+//    if (output_bias_params)
+//        bl.output();
 }
 
 const int BiasLearner::pow4[] = {1,4,16,64};
@@ -207,7 +212,7 @@ void BiasLearner::preProcessTranscript(const Scaffold& transcript)
 	vector<double> startHist(transcript.length()+1, 0.0); // +1 catches overhangs
 	vector<double> endHist(transcript.length()+1, 0.0);
 
-	foreach (const MateHit* hit_p, transcript.mate_hits())
+	BOOST_FOREACH (const MateHit* hit_p, transcript.mate_hits())
 	{
 		const MateHit& hit = *hit_p;
 		if (!hit.left_alignment() && !hit.right_alignment())
@@ -264,10 +269,10 @@ void BiasLearner::processTranscript(const std::vector<double>& startHist, const
 		if (i - min_frag_len > endBinCutoff)
 			endBinCutoff = positionBins[++currEndBin]*(seqLen - min_frag_len);
 			
-		_startPosParams(currStartBin, lenClass) += startHist[i]/fpkm;
-		_startPosExp(currStartBin, lenClass) += !(_frag_len_dist->too_short(seqLen-i));
-		_endPosParams(currEndBin, lenClass) += endHist[i]/fpkm;
-		_endPosExp(currEndBin, lenClass) += !(_frag_len_dist->too_short(i+1));
+		_startPosParams(currStartBin, lenClass) += startHist[i];
+		_startPosExp(currStartBin, lenClass) += fpkm*(_frag_len_dist->cdf(seqLen-i));
+		_endPosParams(currEndBin, lenClass) += endHist[i];
+		_endPosExp(currEndBin, lenClass) += fpkm*(_frag_len_dist->cdf(i+1));
 
 		
 		bool start_in_bounds = i-CENTER >= 0 && i+(_m-1)-CENTER < seqLen;
@@ -287,8 +292,8 @@ void BiasLearner::processTranscript(const std::vector<double>& startHist, const
 				int v = seqToInt(seqSlice,paramTypes[j]);
 				if (v >= 0)
 				{
-					_startSeqParams(j,v) += startHist[i]/fpkm;
-					_startSeqExp(j,v) += !(_frag_len_dist->too_short(seqLen-i));
+					_startSeqParams(j,v) += startHist[i];
+					_startSeqExp(j,v) += fpkm*(_frag_len_dist->cdf(seqLen-i));
 				}
 				else // There is an N.  Average over all possible values of N
 				{
@@ -296,8 +301,8 @@ void BiasLearner::processTranscript(const std::vector<double>& startHist, const
 					genNList(seqSlice, 0, paramTypes[j],nList);
 					for (list<int>::iterator it=nList.begin(); it!=nList.end(); ++it)
 					{
-						_startSeqParams(j,*it) += startHist[i]/(fpkm * (double)nList.size());
-						_startSeqExp(j,*it) += !(_frag_len_dist->too_short(seqLen-i))/(double)nList.size();
+						_startSeqParams(j,*it) += startHist[i]/((double)nList.size());
+						_startSeqExp(j,*it) += fpkm*(_frag_len_dist->cdf(seqLen-i))/(double)nList.size();
 					}
 				}
 			}
@@ -309,8 +314,8 @@ void BiasLearner::processTranscript(const std::vector<double>& startHist, const
 				int v = seqToInt(seqSlice, paramTypes[j]);
 				if (v >= 0)
 				{
-					_endSeqParams(j,v) += endHist[i]/fpkm;
-					_endSeqExp(j,v) += !(_frag_len_dist->too_short(seqLen-i));
+					_endSeqParams(j,v) += endHist[i];
+					_endSeqExp(j,v) += fpkm*(_frag_len_dist->cdf(seqLen-i));
 				}
 				else // There is an N.  Average over all possible values of N
 				{
@@ -318,8 +323,8 @@ void BiasLearner::processTranscript(const std::vector<double>& startHist, const
 					genNList(seqSlice, 0, paramTypes[j], nList);
 					for (list<int>::iterator it=nList.begin(); it!=nList.end(); ++it)
 					{
-						_endSeqParams(j,*it) += endHist[i]/(fpkm * (double)nList.size());
-						_endSeqExp(j,*it) += !(_frag_len_dist->too_short(seqLen-i))/(double)nList.size();
+						_endSeqParams(j,*it) += endHist[i]/((double)nList.size());
+						_endSeqExp(j,*it) += fpkm*(_frag_len_dist->cdf(seqLen-i))/(double)nList.size();
 					}
 				}
 			}
@@ -569,48 +574,43 @@ void BiasLearner::normalizeParameters()
 	}
 }
 
-void BiasLearner::output()
+void BiasLearner::output(FILE* output_file, const string& condition_name, int replicate_num) const
 {
-	ofstream myfile1;
-	string filename = output_dir + "/biasParams.csv";
-	myfile1.open (filename.c_str());
-	
 	// StartSeq
 	for (int i = 0; i < _n; ++i)
 	{
 		for(int j = 0; j < _m; ++j)
-			myfile1 << _startSeqParams(j,i) <<",";
-		myfile1 << endl;
+        {
+			fprintf(output_file, "%s\t%d\tstart_seq\t%d\t%d\t%Lg\n",condition_name.c_str(), replicate_num, i, j,  _startSeqParams(j,i));
+        }
 	}
-	myfile1 << endl;
-	
-	// EndSeq
-	for (int i = 0; i < _n; ++i)
-	{
-		for(int j = 0; j < _m; ++j)
-			myfile1 << _endSeqParams(j,i) <<",";
-		myfile1 << endl;
-	}
-	myfile1 << endl;
-	
+
+    // EndSeq
+    for (int i = 0; i < _n; ++i)
+    {
+        for(int j = 0; j < _m; ++j)
+        {
+            fprintf(output_file, "%s\t%d\tend_seq\t%d\t%d\t%Lg\n",condition_name.c_str(), replicate_num, i, j,  _startSeqParams(j,i));
+        }
+    }
+
 	// Start Pos
 	for (size_t i = 0; i < _startPosParams.size2(); ++i)
 	{
 		for(size_t j = 0; j < _startPosParams.size1(); ++j)
-			myfile1 << _startPosParams(j,i) <<",";
-		myfile1 <<endl;
+        {
+			fprintf(output_file, "%s\t%d\tstart_pos\t%lu\t%lu\t%Lg\n",condition_name.c_str(), replicate_num, i, j,  _startPosParams(j,i));
+        }
 	}
-	myfile1 << endl;	
-	
+
 	// End Pos
 	for (size_t i = 0; i < _endPosParams.size2(); ++i)
 	{
 		for(size_t j = 0; j < _endPosParams.size1(); ++j)
-			myfile1 << _endPosParams(j,i) <<",";
-		myfile1 <<endl;
+        {
+			fprintf(output_file, "%s\t%d\tend_pos\t%lu\t%lu\t%Lg\n",condition_name.c_str(), replicate_num, i, j,  _endPosParams(j,i));
+        }
 	}
-	
-	myfile1.close();
 }
 
 
@@ -803,6 +803,11 @@ double BiasCorrectionHelper::get_effective_length()
 	if (_size==0)
 		return _transcript->length();
 
+    if (no_effective_length_correction)
+        return _transcript->length();
+    
+    if (no_length_correction)
+        return 1;
 	
 	double tot_mass = accumulate( _rg_masses.begin(), _rg_masses.end(), 0.0 );
 	double eff_len = 0.0;
