@@ -228,7 +228,7 @@ void load_ref_rnas(FILE* ref_mRNA_file,
 			}
 		}
         
-        foreach (shared_ptr<Scaffold> s, ref_mRNAs)
+        BOOST_FOREACH (shared_ptr<Scaffold> s, ref_mRNAs)
         {
             assert (s);
         }
@@ -283,7 +283,9 @@ bool unmapped_hit(const MateHit& x)
 bool HitBundle::add_open_hit(shared_ptr<ReadGroupProperties const> rg_props,
                              const ReadHit* bh,
 							 bool expand_by_partner)
-{	
+{
+    assert (bh != NULL);
+    
 	_leftmost = min(_leftmost, bh->left());
 	_ref_id = bh->ref_id();
     
@@ -409,6 +411,7 @@ bool HitBundle::add_open_hit(shared_ptr<ReadGroupProperties const> rg_props,
 void HitBundle::collapse_hits()
 {
 	::collapse_hits(_hits, _non_redundant);
+    //_non_redundant = _hits;
 }
 
 void HitBundle::finalize_open_mates()
@@ -418,7 +421,7 @@ void HitBundle::finalize_open_mates()
 
     for(OpenMates::iterator itr = _open_mates.begin(); itr != _open_mates.end(); ++itr)
     {
-        foreach (MateHit& hit,  itr->second)
+        BOOST_FOREACH (MateHit& hit,  itr->second)
         {
             delete hit.left_alignment();
             delete hit.right_alignment();
@@ -435,30 +438,7 @@ void HitBundle::remove_hitless_scaffolds()
 	_ref_scaffs.erase(new_end, _ref_scaffs.end());	
 }
 
-void HitBundle::remove_unmapped_hits()
-{
-	
-	foreach (MateHit& hit, _hits)
-	{
-		if (unmapped_hit(hit))
-		{
-			delete hit.left_alignment();
-			delete hit.right_alignment();
-		} 
-	}
-	
-	vector<MateHit>::iterator new_end = remove_if(_hits.begin(),
-												  _hits.end(),
-												  unmapped_hit);
 
-	_hits.erase(new_end, _hits.end());	
-	
-	new_end = remove_if(_non_redundant.begin(),
-						_non_redundant.end(),
-						unmapped_hit);
-	_non_redundant.erase(new_end, _non_redundant.end());
-
-}
 
 void HitBundle::combine(const vector<HitBundle*>& in_bundles,
                         HitBundle& out_bundle)
@@ -576,9 +556,35 @@ void HitBundle::combine(const vector<HitBundle*>& in_bundles,
 void HitBundle::finalize(bool is_combined)
 {
 	_final = true;
-    
-	if (!is_combined)
+    if (!is_combined)
 	{
+        // only perform read skipping on primary bundles 
+        // (i.e. don't do it on bundles we're making by combining two or more other bundles)
+        size_t num_skipped = _hits.size() * read_skip_fraction;
+        if (num_skipped > 0 && num_skipped < _hits.size())
+        {
+            random_shuffle(_hits.begin(), _hits.end());
+            for (int i = (int)_hits.size() - num_skipped; i >= 0 && i < (int)_hits.size(); ++i)
+            {
+                delete _hits[i].left_alignment();
+                _hits[i].left_alignment(NULL);
+                
+                delete _hits[i].right_alignment();
+                _hits[i].right_alignment(NULL);
+            }
+            _hits.resize(_hits.size() - num_skipped);
+            is_combined = false;
+        }
+        else if (num_skipped >= _hits.size())
+        {
+            for (size_t i = 0; i < _hits.size(); ++i)
+            {
+                delete _hits[i].left_alignment();
+                delete _hits[i].right_alignment();
+            }
+            _hits.clear();
+        }
+
 		sort(_hits.begin(), _hits.end(), mate_hit_lt);
         if (cond_prob_collapse)
         {
@@ -586,9 +592,9 @@ void HitBundle::finalize(bool is_combined)
         }
         else
         {
-            foreach (MateHit& hit, _hits)
+            BOOST_FOREACH (MateHit& hit, _hits)
             {
-                hit.incr_collapse_mass(hit.common_scale_mass());
+                hit.incr_collapse_mass(hit.internal_scale_mass());
             }
             _non_redundant = _hits;
             
@@ -631,7 +637,7 @@ void HitBundle::finalize(bool is_combined)
 		}
         if (hit.is_mapped())
         {
-            _compatible_mass += hit.mass();
+            _compatible_mass += hit.internal_scale_mass();
         }
 	}
     
@@ -733,8 +739,6 @@ double BundleFactory::next_valid_alignment(const ReadHit*& bh)
         {
             if ((*next_mask_scaff)->left() >= tmp.left())
             {
-                //REMOVE ME:
-                int a = 4;
                 break;
             }
             
@@ -789,19 +793,26 @@ bool BundleFactory::next_bundle_hit_driven(HitBundle& bundle)
         
         // If we are randomly throwing out reads, check to see
         // whether this one should be kept.
-        if (read_skip_fraction > 0.0 && _zeroone() < read_skip_fraction ||
-            bundle.hits().size() >= max_frags_per_bundle)
+        if (bundle.hits().size() >= max_frags_per_bundle)
         {
             skip_read = true;
             next_valid_alignment(bh);
         }
         else
         {
-            bundle.add_raw_mass(next_valid_alignment(bh));
+            double raw_mass = next_valid_alignment(bh);
+            if (bh && bh->num_hits() > max_frag_multihits)
+            {
+                skip_read = true;
+            }
+            else
+            {
+                bundle.add_raw_mass(raw_mass);
+            }
         }
 	}
 	
-	if (skip_read || !bundle.add_open_hit(read_group_properties(), bh))
+	if ((skip_read || !bundle.add_open_hit(read_group_properties(), bh)) && bh != NULL)
     {
         delete bh;
         bh = NULL;
@@ -823,19 +834,33 @@ bool BundleFactory::next_bundle_ref_driven(HitBundle& bundle)
 		const ReadHit* bh = NULL;
 		while(_hit_fac->records_remain())
 		{
-            if (read_skip_fraction == 0.0 || _zeroone() >= read_skip_fraction ||
-                bundle.hits().size() >= max_frags_per_bundle)
+            double raw_mass = next_valid_alignment(bh);
+            if (bundle.hits().size() < max_frags_per_bundle)
             {
-                bundle.add_raw_mass(next_valid_alignment(bh));
+                if (bh && bh->num_hits() > max_frag_multihits)
+                {
+                    
+                }
+                else
+                {
+                    bundle.add_raw_mass(raw_mass);
+                }
+                if (bh) { delete bh; }
             }
+            else
+            {
+                delete bh;
+                bh = NULL;
+            }
+		    
 		}
-        bundle.finalize();
+		bundle.finalize();
 		return false;
 	}
 	
 	bundle.add_ref_scaffold(*next_ref_scaff);
-	next_ref_scaff++;
-		
+	++next_ref_scaff;
+    
 	_expand_by_refs(bundle);
 	
 	// The most recent RefID and position we've seen in the hit stream
@@ -850,18 +875,24 @@ bool BundleFactory::next_bundle_ref_driven(HitBundle& bundle)
         bool skip_read = false;
 		// If we are randomly throwing out reads, check to see
         // whether this one should be kept.
-        double t = _zeroone();
-        if (read_skip_fraction > 0.0 && _zeroone() < read_skip_fraction ||
-            bundle.hits().size() >= max_frags_per_bundle)
+        if (bundle.hits().size() >= max_frags_per_bundle)
         {
             next_valid_alignment(bh);
             skip_read = true;
         }
         else
         {
-            bundle.add_raw_mass(next_valid_alignment(bh));
+            double raw_mass = next_valid_alignment(bh);
+            if (bh && bh->num_hits() > max_frag_multihits)
+            {
+                skip_read = true;
+            }
+            else
+            {
+                bundle.add_raw_mass(raw_mass);
+            }
         }
-
+        
         if (bh == NULL)
         {
 			if (_hit_fac->records_remain())
@@ -869,7 +900,7 @@ bool BundleFactory::next_bundle_ref_driven(HitBundle& bundle)
 			else
 				break;
         }
-				
+        
 		last_hit_ref_id_seen = bh->ref_id();
 		last_hit_pos_seen = bh->left();
 		
@@ -882,18 +913,19 @@ bool BundleFactory::next_bundle_ref_driven(HitBundle& bundle)
 			if (bh_chr_order < bundle_chr_order) // the hit stream has not caught up, skip
 			{
 				delete bh;
+                bh = NULL;
 				continue; 
 			}
 			else // the hit stream has gone too far, rewind and break
 			{
-                double mass = rewind_hit(bh);
-                if (skip_read == false)
-                {
-                    bundle.rem_raw_mass(mass);
-                }
-				break;  
+                rewind_hit(bh);
+                bh = NULL;
+                break;
 			}
-		}	
+		}
+        
+        if (bh == NULL) // the hit stream has gone too far, break
+            break;
 		
         if (bh->left() >= bundle.left() && bh->right() <= bundle.right())
 		{
@@ -913,17 +945,30 @@ bool BundleFactory::next_bundle_ref_driven(HitBundle& bundle)
 		}
 		else if (bh->left() >= bundle.right())
 		{
-            if (!skip_read)
+            if (skip_read == false)
             {
                 bundle.rem_raw_mass(rewind_hit(bh));
+                bh = NULL;
+            }
+            else
+            {
+                delete bh;
+                bh = NULL;
             }
 			break;
 		}
-	    else 
+	    else
         {
             // It's not within the bundle bounds, but it's also not past the 
             // right end, so skip it.
             delete bh;
+            bh = NULL;
+        }
+        
+        if (skip_read == true && bh != NULL)
+        {
+            delete bh;
+            bh = NULL;
         }
 	}
 	
@@ -960,7 +1005,8 @@ bool BundleFactory::next_bundle_ref_guided(HitBundle& bundle)
 		int scaff_chr_order = _hit_fac->ref_table().observation_order((*next_ref_scaff)->ref_id());
 		
 		bundle.rem_raw_mass(rewind_hit(bh));
-		
+		bh = NULL;
+        
 		if (bh_chr_order < scaff_chr_order)
 		{
 			return next_bundle_hit_driven(bundle);
@@ -982,6 +1028,8 @@ bool BundleFactory::next_bundle_ref_guided(HitBundle& bundle)
 	else 
 	{
 		bundle.rem_raw_mass(rewind_hit(bh));
+        bh = NULL;
+        
 		bundle.add_ref_scaffold(*next_ref_scaff);
 		next_ref_scaff++;
 		_expand_by_refs(bundle);
@@ -1031,15 +1079,16 @@ bool BundleFactory::_expand_by_hits(HitBundle& bundle)
         bool skip_read = false;
         const ReadHit* bh = NULL;
         
-        if (read_skip_fraction > 0.0 && _zeroone() < read_skip_fraction)
+        double raw_mass = next_valid_alignment(bh);
+        if (bh && bh->num_hits() > max_frag_multihits)
         {
             skip_read = true;
         }
         else
         {
-            bundle.add_raw_mass(next_valid_alignment(bh));
-		}
-        
+            bundle.add_raw_mass(raw_mass);
+        }
+
 		if (bh == NULL)
 		{
 			if (_hit_fac->records_remain())
@@ -1063,6 +1112,7 @@ bool BundleFactory::_expand_by_hits(HitBundle& bundle)
 		else
 		{
 			bundle.rem_raw_mass(rewind_hit(bh));
+
 			break;
 		}
 	}
@@ -1316,7 +1366,7 @@ void identify_bad_splices(const HitBundle& bundle,
 	ins_itr = bad_splice_ops.insert(make_pair(ref_id, vector<AugmentedCuffOp>()));
 	vector<AugmentedCuffOp>& bad_introns = ins_itr.first->second;
 	
-	foreach (const MateHit& hit, bundle.hits())
+	BOOST_FOREACH (const MateHit& hit, bundle.hits())
 	{
 		if (hit.left_alignment())
 		{
@@ -1541,3 +1591,403 @@ bool BundleFactory::spans_bad_intron(const ReadHit& read)
 	
 	return false;
 }
+
+void inspect_map(BundleFactory& bundle_factory,
+                 BadIntronTable* bad_introns,
+                 vector<LocusCount>& compatible_count_table,
+                 vector<LocusCount>& total_count_table,
+                 bool progress_bar,
+                 bool show_stats)
+{
+
+	ProgressBar p_bar;
+	if (progress_bar)
+		p_bar = ProgressBar("Inspecting reads and determining fragment length distribution.",bundle_factory.ref_table().size());
+	RefID last_chrom = 0;
+
+	long double map_mass = 0.0;
+    long double norm_map_mass = 0.0;
+	
+	int min_len = numeric_limits<int>::max();
+	int max_len = def_max_frag_len;
+	vector<double> frag_len_hist(def_max_frag_len+1,0);
+	bool has_pairs = false;
+
+	int num_bundles = 0;
+	size_t total_hits = 0;
+	size_t total_non_redundant_hits = 0;
+	
+	//To be used for quartile normalization
+	vector<long double> mass_dist; 	
+	
+	// Store the maximum read length for "first" and "second" reads to report to user.
+	int max_1 = 0;
+	int max_2 = 0;
+	
+	shared_ptr<MultiReadTable> mrt(new MultiReadTable());
+	
+	while(true)
+	{
+		HitBundle* bundle_ptr = new HitBundle();
+		
+		bool valid_bundle = bundle_factory.next_bundle(*bundle_ptr);
+		HitBundle& bundle = *bundle_ptr;
+
+        if (use_compat_mass) //only count hits that are compatible with ref transcripts
+        {
+            // Take raw mass even if bundle is "empty", since we could be out of refs
+            // with remaining hits
+            map_mass += bundle.compatible_mass();
+//            if (lib_norm_method == QUARTILE && bundle.compatible_mass() > 0)
+//            {
+//                mass_dist.push_back(bundle.compatible_mass());
+//            }
+        }
+        else if (use_total_mass) //use all raw mass
+        { 
+            
+            // Take raw mass even if bundle is "empty", since we could be out of refs
+            // with remaining hits
+            map_mass += bundle.raw_mass();
+//            if (lib_norm_method == QUARTILE && bundle.raw_mass() > 0)
+//            {
+//                mass_dist.push_back(bundle.raw_mass());
+//            }
+        }
+        else
+        {
+            fprintf(stderr, "Error: hit counting scheme for normalization is not set!\n");
+            assert(false);
+            exit(1);
+        }
+		
+		const RefSequenceTable& rt = bundle_factory.ref_table();
+		const char* chrom = rt.get_name(bundle.ref_id());
+		char bundle_label_buf[2048];
+        if (chrom)
+        {
+            sprintf(bundle_label_buf, "%s:%d-%d", chrom, bundle.left(), bundle.right());
+            verbose_msg("Inspecting bundle %s with %lu reads\n", bundle_label_buf, bundle.hits().size());
+            compatible_count_table.push_back(LocusCount(bundle_label_buf, floor(bundle.compatible_mass()), bundle.ref_scaffolds().size()));
+            total_count_table.push_back(LocusCount(bundle_label_buf, floor(bundle.raw_mass()), bundle.ref_scaffolds().size()));
+		}
+        
+        if (!valid_bundle)
+		{
+			delete bundle_ptr;
+			break;
+		}
+		num_bundles++;
+        
+        if (progress_bar) 
+        {
+			double inc_amt = last_chrom == bundle.ref_id() ? 0.0 : 1.0;
+			p_bar.update(bundle_label_buf, inc_amt);
+			last_chrom = bundle.ref_id();
+        }
+        
+        if (bad_introns != NULL)
+		{
+			identify_bad_splices(bundle, *bad_introns);
+		}
+		
+		const vector<MateHit>& hits = bundle.non_redundant_hits();
+		if (hits.empty())
+		{
+			delete bundle_ptr;
+			continue;
+		}
+		
+		list<pair<int, int> > open_ranges;
+		int curr_range_start = hits[0].left();
+		int curr_range_end = numeric_limits<int>::max();
+		int next_range_start = -1;
+		
+		total_non_redundant_hits += bundle.non_redundant_hits().size();
+		total_hits += bundle.hits().size();
+		
+		// This first loop calclates the map mass and finds ranges with no introns
+		// Note that we are actually looking at non-redundant hits, which is why we use collapse_mass
+		// This loop will also add multi-reads to the MultiReads table 
+		for (size_t i = 0; i < hits.size(); ++i) 
+		{
+			assert(hits[i].left_alignment());
+            
+            // Add to table if multi-read
+			if (hits[i].is_multi())
+			{
+				mrt->add_hit(hits[i]);
+			}
+			
+			// Find left length
+			int left_len = hits[i].left_alignment()->right()-hits[i].left_alignment()->left();
+			min_len = min(min_len, left_len);
+			if (!hits[i].left_alignment()->contains_splice())
+            {
+				if (hits[i].left_alignment()->is_first())
+                    max_1 = max(max_1, left_len);
+                else
+                    max_2 = max(max_2, left_len);
+            }
+			
+			// Find right length
+			if (hits[i].right_alignment())
+			{
+				int right_len = hits[i].right_alignment()->right()-hits[i].right_alignment()->left();
+				min_len = min(min_len, right_len);
+				if (!hits[i].right_alignment()->contains_splice())
+                {
+                    if (hits[i].right_alignment()->is_first())
+                        max_1 = max(max_1, right_len);
+                    else
+                        max_2 = max(max_2, right_len);
+                }
+                has_pairs = true;
+			}
+			
+			// Find fragment length
+			if (bundle.ref_scaffolds().size()==1 && hits[i].is_pair())
+			// Annotation provided and single isoform gene
+			{
+				int start, end, mate_length;
+				shared_ptr<Scaffold> scaff = bundle.ref_scaffolds()[0];
+				if (scaff->map_frag(hits[i], start, end, mate_length))
+				{
+					if (mate_length >= min_len && mate_length <= max_len)
+						frag_len_hist[mate_length] += hits[i].collapse_mass();
+				}
+			}
+			else if (bundle.ref_scaffolds().empty())
+			// No annotation provided.  Look for ranges.
+			{
+				if (hits[i].left() > curr_range_end)
+				{
+					if (curr_range_end - curr_range_start > max_len)
+						open_ranges.push_back(make_pair(curr_range_start, curr_range_end));
+					curr_range_start = next_range_start;
+					curr_range_end = numeric_limits<int>::max();
+				}
+				if (hits[i].left_alignment()->contains_splice())
+				{
+					if (hits[i].left() - curr_range_start > max_len)
+						open_ranges.push_back(make_pair(curr_range_start, hits[i].left()-1));
+					curr_range_start = max(next_range_start, hits[i].left_alignment()->right());
+				}
+				if (hits[i].right_alignment() && hits[i].right_alignment()->contains_splice())
+				{
+					assert(hits[i].right_alignment()->left() >= hits[i].left());
+					curr_range_end = min(curr_range_end, hits[i].right_alignment()->left()-1);
+					next_range_start = max(next_range_start, hits[i].right());
+				}
+			}
+		}
+        
+        if (bundle.ref_scaffolds().empty() && has_pairs) // No annotation provided
+		{
+			pair<int, int> curr_range(-1,-1);
+			
+			// This second loop uses the ranges found above to find the estimated frag length distribution
+			// It also finds the minimum read length to use in the linear interpolation
+			for (size_t i = 0; i < hits.size(); ++i)
+			{
+				if (hits[i].left() > curr_range.second && open_ranges.empty())
+					break;
+				
+				if (hits[i].left() > curr_range.second)
+				{
+					curr_range = open_ranges.front();
+					open_ranges.pop_front();
+				}
+				
+				if (hits[i].left() >= curr_range.first && hits[i].right() <= curr_range.second && hits[i].is_pair())
+				{
+					int mate_len = hits[i].right()-hits[i].left();
+					if (mate_len <= max_len)
+						frag_len_hist[mate_len] += hits[i].collapse_mass();
+				}
+			}
+		}
+		
+        open_ranges.clear();
+		delete bundle_ptr;
+	}
+	
+    norm_map_mass = map_mass;
+    
+//	if (lib_norm_method == QUARTILE && mass_dist.size() > 0)
+//	{
+//		sort(mass_dist.begin(),mass_dist.end());
+//		int upper_quart_index = mass_dist.size() * 0.75;
+//		norm_map_mass = mass_dist[upper_quart_index];
+//	}
+
+    if (bad_introns != NULL)
+    {
+        size_t alloced = 0;
+        size_t used = 0;
+        size_t num_introns = 0;
+        for (BadIntronTable::const_iterator itr = bad_introns->begin();
+             itr != bad_introns->end();
+             ++itr)
+        {
+            alloced += itr->second.capacity() * sizeof(AugmentedCuffOp);
+            used += itr->second.size() * sizeof(AugmentedCuffOp);
+            num_introns += itr->second.size();
+        }
+        
+        verbose_msg( "Bad intron table has %lu introns: (%lu alloc'd, %lu used)\n", num_introns, alloced, used);
+    	verbose_msg( "Map has %lu hits, %lu are non-redundant\n", total_hits, total_non_redundant_hits);
+    } 
+    
+	if (progress_bar)
+		p_bar.complete();
+	
+	vector<double> frag_len_pdf(max_len+1, 0.0);
+	vector<double> frag_len_cdf(max_len+1, 0.0);
+    long double tot_count = accumulate(frag_len_hist.begin(), frag_len_hist.end(), 0.0 );
+    bool empirical = false;
+	
+	if (user_provided_fld && has_pairs && tot_count >= 10000)
+	{
+		fprintf(stderr, "Warning: Overriding empirical fragment length distribution with user-specified parameters is not recommended.\n");
+	}
+	
+	if (!has_pairs || tot_count < 10000)
+	{
+		if (has_pairs && !user_provided_fld)
+		{
+			fprintf(stderr, "Warning: Using default Gaussian distribution due to insufficient paired-end reads in open ranges.  It is recommended that correct parameters (--frag-len-mean and --frag-len-std-dev) be provided.\n");
+		}
+		tot_count = 0;
+		normal frag_len_norm(def_frag_len_mean, def_frag_len_std_dev);
+		max_len = def_frag_len_mean + 3*def_frag_len_std_dev;
+		for(int i = min_len; i <= max_len; i++)
+		{
+			frag_len_hist[i] = cdf(frag_len_norm, i+0.5)-cdf(frag_len_norm, i-0.5);
+			tot_count += frag_len_hist[i];
+		}
+	}
+	else
+	// Calculate the max frag length and interpolate all zeros between min read len and max frag len
+	{	
+		empirical = true;
+		double curr_total = 0;
+		size_t last_nonzero = min_len-1;
+		for(size_t i = last_nonzero+1; i < frag_len_hist.size(); i++)
+		{
+			if (frag_len_hist[i] > 0)
+			{
+				if (last_nonzero != i-1)
+				{
+					double b = frag_len_hist[last_nonzero];
+					double m = (frag_len_hist[i] - b)/(i-last_nonzero);
+					for (size_t x = 1; x < i - last_nonzero; x++)
+					{
+						frag_len_hist[last_nonzero+x] = m * x + b;
+						tot_count += frag_len_hist[last_nonzero+x];
+						curr_total += frag_len_hist[last_nonzero+x];
+					}	
+				}
+				last_nonzero = i;
+			}
+			
+			curr_total += frag_len_hist[i];
+			
+			if (curr_total/tot_count > 0.9999)
+			{
+				max_len = i; 
+				tot_count = curr_total;
+				break;
+			}
+		}
+	}
+	
+    double mean = 0.0;
+
+    if (output_fld)
+    {
+        FILE* fhist = fopen(string(output_dir + "/frag_len_hist.csv").c_str(),"w");
+        fprintf(fhist, "Length,Count\n");
+        for(size_t i = 1; i < frag_len_hist.size(); i++)
+        {
+            fprintf(fhist, "%zu,%f\n", i, frag_len_hist[i]);
+        }
+        fclose(fhist);
+    }
+
+	// Convert histogram to pdf and cdf, calculate mean
+	int frag_len_mode = 0;
+	for(size_t i = min_len; i <= (size_t)max_len; i++)
+	{
+		frag_len_pdf[i] = frag_len_hist[i]/tot_count;
+		frag_len_cdf[i] = frag_len_cdf[i-1] + frag_len_pdf[i];
+        
+		if (frag_len_pdf[i] > frag_len_pdf[frag_len_mode])
+			frag_len_mode = i;
+        mean += frag_len_pdf[i] * i;
+	}
+    
+    double std_dev =  0.0;
+    for(size_t i = 1; i < frag_len_hist.size(); i++)
+    {
+        std_dev += frag_len_pdf[i] * ((i - mean) * (i - mean));
+    }
+    
+    std_dev = sqrt(std_dev);
+	
+	shared_ptr<ReadGroupProperties> rg_props = bundle_factory.read_group_properties();
+
+    FLDSource source = DEFAULT;
+    if (empirical)
+    {
+        source = LEARNED;
+    }
+    else if (user_provided_fld)
+    {
+        source = USER;
+    }
+
+	shared_ptr<EmpDist const> fld(new EmpDist(frag_len_pdf, frag_len_cdf, frag_len_mode, mean, std_dev, min_len, max_len, source));
+	rg_props->multi_read_table(mrt);
+	rg_props->frag_len_dist(fld);
+	rg_props->normalized_map_mass(norm_map_mass);
+    rg_props->total_map_mass(map_mass);
+
+    if (show_stats)
+    {
+        fprintf(stderr, "> Map Properties:\n");
+        //if (lib_norm_method == QUARTILE)
+        //    fprintf(stderr, ">\tUpper Quartile: %.2Lf\n", norm_map_mass);
+        fprintf(stderr, ">\tNormalized Map Mass: %.2Lf\n", norm_map_mass);
+        fprintf(stderr, ">\tRaw Map Mass: %.2Lf\n", map_mass);
+        if (corr_multi)
+            fprintf(stderr,">\tNumber of Multi-Reads: %zu (with %zu total hits)\n", mrt->num_multireads(), mrt->num_multihits()); 
+    //	if (has_pairs)
+    //		fprintf(stderr, ">\tRead Type: %dbp x %dbp\n", max_1, max_2);
+    //	else
+    //		fprintf(stderr, ">\tRead Type: %dbp single-end\n", max(max_1,max_2));
+
+        if (empirical)
+        {
+            fprintf(stderr, ">\tFragment Length Distribution: Empirical (learned)\n");
+            fprintf(stderr, ">\t              Estimated Mean: %.2f\n", mean);
+            fprintf(stderr, ">\t           Estimated Std Dev: %.2f\n", std_dev);
+        }
+        else
+        {
+            if (user_provided_fld)
+            {
+                fprintf(stderr, ">\tFragment Length Distribution: Truncated Gaussian (user-specified)\n");
+            }
+            else
+            {
+                fprintf(stderr, ">\tFragment Length Distribution: Truncated Gaussian (default)\n");
+            }
+            fprintf(stderr, ">\t              Default Mean: %d\n", def_frag_len_mean);
+            fprintf(stderr, ">\t           Default Std Dev: %d\n", def_frag_len_std_dev);
+        }
+    }
+	bundle_factory.num_bundles(num_bundles);
+	bundle_factory.reset(); 
+	return;
+}
