@@ -22,8 +22,6 @@
 using boost::math::normal;
 
 #include <boost/foreach.hpp>
-#define foreach         BOOST_FOREACH
-#define reverse_foreach BOOST_REVERSE_FOREACH
 
 #include <boost/thread.hpp>
 #include <boost/shared_ptr.hpp>
@@ -49,6 +47,7 @@ extern int max_partner_dist;
 extern uint32_t max_gene_length;
 extern std::string ref_gtf_filename;
 extern std::string mask_gtf_filename;
+extern std::string contrast_filename;
 extern std::string output_dir;
 extern std::string fasta_dir;
 extern std::string library_type;
@@ -56,17 +55,16 @@ extern std::string library_type;
 // Abundance estimation options
 extern bool corr_bias;
 extern bool corr_multi;
-extern bool use_quartile_norm;
-extern bool poisson_dispersion;
+
 extern int def_frag_len_mean;
 extern int def_frag_len_std_dev;
 extern int max_mle_iterations;
 extern int num_importance_samples;
 extern float min_isoform_fraction;
-extern bool use_em;
 extern bool cond_prob_collapse;
 extern bool use_compat_mass;
 extern bool use_total_mass;
+extern bool model_mle_error;
 
 // Ref-guided assembly options
 extern int overhang_3;
@@ -99,6 +97,25 @@ extern int num_bootstrap_samples;
 extern double bootstrap_fraction;
 extern double bootstrap_delta_gap;
 extern int max_frags_per_bundle;
+//extern bool analytic_diff;
+extern bool no_differential;
+extern double num_frag_count_draws;
+extern double num_frag_assignments;
+extern double max_multiread_fraction;
+extern double max_frag_multihits;
+extern int min_reps_for_js_test;
+extern bool no_effective_length_correction;
+extern bool no_length_correction;
+extern bool no_js_tests;
+
+extern bool no_scv_correction;
+
+extern double min_outlier_p;
+
+
+extern std::string default_dispersion_method;
+extern std::string default_lib_norm_method;
+extern std::string default_cufflinks_lib_norm_method;
 
 // SECRET OPTIONS: 
 // These options are just for instrumentation and benchmarking code
@@ -119,6 +136,11 @@ extern boost::thread_specific_ptr<std::string> bundle_label; // for consistent,
 extern boost::shared_ptr<std::string> bundle_label;
 #endif
 
+// Global switch to mark when we're in the middle of learning bias.
+extern bool bias_run;
+
+// Hold the command line string used to run the program
+extern std::string cmd_str;
 
 bool gaurd_assembly();
 
@@ -204,6 +226,32 @@ enum Platform
     SOLID
 };
 
+enum FLDSource
+{
+    LEARNED,
+    USER,
+    DEFAULT
+};
+
+enum DispersionMethod
+{
+    DISP_NOT_SET,
+    BLIND,
+    PER_CONDITION,
+    POOLED,
+    POISSON
+};
+
+enum LibNormalizationMethod
+{
+    LIB_NORM_NOT_SET,
+    GEOMETRIC,
+    CLASSIC_FPKM,
+    TMM,
+    QUARTILE,
+    ABSOLUTE // Requires spike-in controls, not yet implemented
+};
+
 class EmpDist
 {
 	//Vectors only valid between min and max!
@@ -214,10 +262,11 @@ class EmpDist
     double _std_dev;
 	int _min;
 	int _max;
-	
+	FLDSource _source;
+    
 public:
-	EmpDist(std::vector<double>& pdf, std::vector<double>& cdf, int mode, double mean, double std_dev, int min, int max)
-	: _pdf(pdf), _cdf(cdf), _mode(mode), _mean(mean), _std_dev(std_dev), _min(min), _max(max) {}
+	EmpDist(std::vector<double>& pdf, std::vector<double>& cdf, int mode, double mean, double std_dev, int min, int max, FLDSource source)
+	: _pdf(pdf), _cdf(cdf), _mode(mode), _mean(mean), _std_dev(std_dev), _min(min), _max(max), _source(source) {}
 	
 	void pdf(std::vector<double>& pdf)	{ _pdf = pdf; }
 	double pdf(int l) const
@@ -266,12 +315,16 @@ public:
     
     void std_dev(double std_dev)				{ _std_dev = std_dev;  }
 	double std_dev() const					{ return _std_dev; }
+    
+    FLDSource source() const        { return _source; }
+    void source(FLDSource source)   { _source = source; } 
 };
 
 class BiasLearner;
 class MultiReadTable;
 
 class MassDispersionModel;
+class MleErrorModel;
 
 struct LocusCount
 {
@@ -312,21 +365,27 @@ public:
 	boost::shared_ptr<BiasLearner const> bias_learner() const { return _bias_learner; }
     void bias_learner(boost::shared_ptr<BiasLearner const> bl)  { _bias_learner = bl; } 
 	
-    void mass_scale_factor(double sf) { _mass_scaling_factor = sf; }
-    double mass_scale_factor() const  { return _mass_scaling_factor; }
+    // The internal scaling factor relates replicates to each other, so
+    // that replicates with larger library sizes don't bias the isoform
+    // deconvolution over smaller libraries
+    void internal_scale_factor(double sf) { _internal_scale_factor = sf; }
+    double internal_scale_factor() const  { return _internal_scale_factor; }
+    
+    void external_scale_factor(double sf) { _external_scale_factor = sf; }
+    double external_scale_factor() const  { return _external_scale_factor; }
     
     void complete_fragments(bool c)  { _complete_fragments = c; }
     bool complete_fragments() const { return _complete_fragments; }
     
-    double scale_mass(double unscaled_mass) const 
+    double internally_scale_mass(double unscaled_mass) const 
     { 
-        if (_mass_scaling_factor == 0)
+        if (_internal_scale_factor == 0)
             return unscaled_mass;
         
-        return unscaled_mass * (1.0 / _mass_scaling_factor);
+        return unscaled_mass * (1.0 / _internal_scale_factor);
     }
     
-    boost::shared_ptr<const MassDispersionModel> mass_dispersion_model() const 
+    boost::shared_ptr<const MassDispersionModel> mass_dispersion_model() const
     { 
         return _mass_dispersion_model; 
     };
@@ -336,12 +395,43 @@ public:
         _mass_dispersion_model = nm; 
     }
     
-    const std::vector<LocusCount>& common_scale_counts() { return _common_scale_counts; }
-    void common_scale_counts(const std::vector<LocusCount>& counts) { _common_scale_counts = counts; }
+    boost::shared_ptr<const MleErrorModel> mle_error_model() const
+    {
+        return _mle_error_model;
+    };
+    
+    void mle_error_model(boost::shared_ptr<const MleErrorModel> nm)
+    {
+        _mle_error_model = nm;
+    }
+    
+    const std::vector<LocusCount>& common_scale_compatible_counts() { return _common_scale_compatible_counts; }
+    void common_scale_compatible_counts(const std::vector<LocusCount>& counts) { _common_scale_compatible_counts = counts; }
+    
+    const std::vector<LocusCount>& common_scale_total_counts() { return _common_scale_total_counts; }
+    void common_scale_total_counts(const std::vector<LocusCount>& counts) { _common_scale_total_counts = counts; }
+    
+    const std::vector<LocusCount>& raw_compatible_counts() { return _raw_compatible_counts; }
+    void raw_compatible_counts(const std::vector<LocusCount>& counts) { _raw_compatible_counts = counts; }
+    
+    const std::vector<LocusCount>& raw_total_counts() { return _raw_total_counts; }
+    void raw_total_counts(const std::vector<LocusCount>& counts) { _raw_total_counts = counts; }
     
 	boost::shared_ptr<MultiReadTable> multi_read_table() const {return _multi_read_table; }	
 	void multi_read_table(boost::shared_ptr<MultiReadTable> mrt) { _multi_read_table = mrt;	}
 	
+//    const string& description() const { return _description; }
+//    void description(const string& d) { _description = d; }
+    
+    const std::string& condition_name() const { return _condition_name; }
+    void condition_name(const std::string& cd) { _condition_name = cd; }
+    
+    const std::string& file_path() const { return _file_path; }
+    void file_path(const std::string& fp) { _file_path = fp; }
+    
+    int replicate_num() const { return _replicate_num; }
+    void replicate_num(int rn) { _replicate_num = rn; }
+    
 private:
     
     Strandedness _strandedness;
@@ -354,20 +444,43 @@ private:
 	boost::shared_ptr<BiasLearner const> _bias_learner;
 	boost::shared_ptr<MultiReadTable> _multi_read_table;
     
-    double _mass_scaling_factor;
+    double _internal_scale_factor;
+    double _external_scale_factor;
     boost::shared_ptr<const MassDispersionModel> _mass_dispersion_model;
-    std::vector<LocusCount> _common_scale_counts;
+    std::vector<LocusCount> _common_scale_compatible_counts;
+    std::vector<LocusCount> _common_scale_total_counts;
+    std::vector<LocusCount> _raw_compatible_counts;
+    std::vector<LocusCount> _raw_total_counts;
+
+    boost::shared_ptr<const MleErrorModel> _mle_error_model;
     
     bool _complete_fragments;
+    
+    std::string _condition_name;
+    std::string _file_path;
+    int _replicate_num;
 };
 
 extern std::map<std::string, ReadGroupProperties> library_type_table;
 
 extern const ReadGroupProperties* global_read_properties;
 
+extern std::map<std::string, DispersionMethod> dispersion_method_table;
+extern DispersionMethod dispersion_method;
+
+extern std::map<std::string, LibNormalizationMethod> lib_norm_method_table;
+extern LibNormalizationMethod lib_norm_method;
+
 void print_library_table();
 void init_library_table();
 
+void print_dispersion_method_table();
+void init_dispersion_method_table();
+
+void print_lib_norm_method_table();
+void init_lib_norm_method_table();
+void init_cufflinks_lib_norm_method_table();
+
 
 template<typename T>
 std::string cat_strings(const T& container, const char* delimiter=",")
@@ -433,4 +546,23 @@ std::string cat_strings(const T& container, const char* delimiter=",")
 #define OPT_TRIM_READ_LENGTH        297
 #define OPT_MAX_DELTA_GAP           298
 #define OPT_MLE_MIN_ACC             299
+//#define OPT_ANALYTIC_DIFF           300
+#define OPT_NO_DIFF                 301
+#define OPT_GEOMETRIC_NORM          302
+#define OPT_RAW_MAPPED_NORM         303
+#define OPT_NUM_FRAG_COUNT_DRAWS    304
+#define OPT_NUM_FRAG_ASSIGN_DRAWS   305
+#define OPT_MAX_MULTIREAD_FRACTION  306
+#define OPT_LOCUS_COUNT_DISPERSION  307
+#define OPT_MIN_OUTLIER_P           308
+#define OPT_FRAG_MAX_MULTIHITS      309
+#define OPT_MIN_REPS_FOR_JS_TEST    310
+#define OPT_OLAP_RADIUS             311
+#define OPT_NO_LENGTH_CORRECTION    312
+#define OPT_NO_EFFECTIVE_LENGTH_CORRECTION    313
+#define OPT_NO_JS_TESTS             314
+#define OPT_DISPERSION_METHOD       315
+#define OPT_LIB_NORM_METHOD         316
+#define OPT_NO_SCV_CORRECTION       317
+
 #endif
