@@ -9,6 +9,10 @@
  *
  */
 
+#ifdef HEAPROFILE
+#include "gperftools/heap-profiler.h"
+#endif
+
 #include "gff.h"
 #include "GFaSeqGet.h"
 #include "GFastaIndex.h"
@@ -181,11 +185,6 @@ public:
 			return (ovlen==b.ovlen)? betterRef(mrna, b.mrna) : (ovlen>b.ovlen);
 		else return rank<b.rank;
 	}
-    bool operator>(COvLink& b) {
-		if (rank==b.rank)
-			return (ovlen==b.ovlen)? betterRef(b.mrna, mrna) : (ovlen<b.ovlen);
-		else return rank>b.rank;
-	}
     bool operator==(COvLink& b) {
 		return (rank==b.rank && mrna==b.mrna);
 	}
@@ -236,6 +235,7 @@ public:
 	GffObj* eqref; //ref transcript having an ichain match
 	int qset; //qry set index (qfidx), -1 means reference dataset
 	//GffObj* eqnext; //next GffObj in the linked list of matching transfrags
+	bool eqhead;
 	CEqList* eqlist; //keep track of matching transfrags
 	//int eqdata; // flags for EQ list (is it a list head?)
 	// Cufflinks specific data:
@@ -250,9 +250,8 @@ public:
 		locus=l;
 		classcode=0;
 		eqref=NULL;
-		//eqnext=NULL;
+		eqhead=false;
 		eqlist=NULL;
-		//eqdata=0;
 		qset=-2;
 		FPKM=0;
 		conf_lo=0;
@@ -263,41 +262,50 @@ public:
 	~CTData() {
 		ovls.Clear();
 		//if ((eqdata & EQHEAD_TAG)!=0) delete eqlist;
-		if (isEqHead()) delete eqlist;
+		//if (isEqHead()) delete eqlist;
+		if (eqhead) delete eqlist;
 	}
 
   //inline bool eqHead() { return ((eqdata & EQHEAD_TAG)!=0); }
-  bool isEqHead() { 
+ /*  bool isEqHead() {
       if (eqlist==NULL) return false;
       return (eqlist->head==this->mrna);
       }
-    
+  */
   void joinEqList(GffObj* m) { //add list from m
    //list head is set to the transfrag with the lower qset#
   CTData* md=(CTData*)(m->uptr);
   //ASSERT(md);
-  if (eqlist==NULL) {
-     if (md->eqlist!=NULL) {
+  if (eqlist==NULL) { //no eqlist yet for this node
+     if (md->eqlist!=NULL) { //m in an eqlist already
           eqlist=md->eqlist;
           eqlist->Add(this->mrna);
           CTData* md_head_d=(CTData*)(md->eqlist->head->uptr);
-          if (this->qset < md_head_d->qset)
+          if (this->qset < md_head_d->qset) {
                eqlist->head=this->mrna;
-          }
-        else { //m was not in an EQ list
-          //eqlist=new GList<GffObj>((GCompareProc*)cmpByPtr, (GFreeProc*)NULL, true);
+               eqhead=true;
+               md_head_d->eqhead=false;
+               }
+        }
+        else { //m was not in an EQ list either
           eqlist=new CEqList();
           eqlist->Add(this->mrna);
           eqlist->Add(m);
           md->eqlist=eqlist;
-          if (qset<md->qset) eqlist->head=this->mrna;
-                       else  eqlist->head=m;
+          if (qset<md->qset) {
+        	eqlist->head=this->mrna;
+        	eqhead=true;
           }
+          else  {
+        	eqlist->head=m;
+        	md->eqhead=true;
+          }
+        }
       }//no eqlist before
      else { //merge two eqlists
       if (eqlist==md->eqlist) //already in the same eqlist, nothing to do
          return;
-      if (md->eqlist!=NULL) { //copy elements of m's eqlist
+      if (md->eqlist!=NULL) {
         //copy the smaller list into the larger one
         CEqList* srclst, *destlst;
         if (md->eqlist->Count()<eqlist->Count()) {
@@ -312,23 +320,31 @@ public:
            destlst->Add(srclst->Get(i));
            CTData* od=(CTData*)((*srclst)[i]->uptr);
            od->eqlist=destlst;
-           //od->eqdata=od->qset+1;
            }
         this->eqlist=destlst;
         CTData* s_head_d=(CTData*)(srclst->head->uptr);
         CTData* d_head_d=(CTData*)(destlst->head->uptr);
-        if (s_head_d->qset < d_head_d->qset )
-             this->eqlist->head=srclst->head; 
-        delete srclst;
+        if (s_head_d->qset < d_head_d->qset ) {
+             this->eqlist->head=srclst->head;
+             s_head_d->eqhead=true;
+             d_head_d->eqhead=false;
         }
-       else { //md->eqlist==NULL
+        else {
+          s_head_d->eqhead=false;
+          d_head_d->eqhead=true;
+        }
+        delete srclst;
+      }
+      else { //md->eqlist==NULL
         eqlist->Add(m);
         md->eqlist=eqlist;
         CTData* head_d=(CTData*)(eqlist->head->uptr);
-        if (md->qset<head_d->qset)
-            eqlist->head=m;
+        if (md->qset<head_d->qset) {
+          eqlist->head=m;
+          md->eqhead=true;
         }
       }
+    }
   }
 
 	void addOvl(char code,GffObj* target=NULL, int ovlen=0) {
@@ -337,7 +353,6 @@ public:
 	char getBestCode() {
 		return (ovls.Count()>0) ? ovls[0]->code : 0 ;
     }
-	bool operator>(CTData& b) { return (mrna > b.mrna); }
 	bool operator<(CTData& b) { return (mrna < b.mrna); }
 	bool operator==(CTData& b) { return (mrna==b.mrna); }
 };
@@ -346,6 +361,12 @@ class GSuperLocus;
 class GTrackLocus;
 class GXLocus;
 
+class GXSeg : public GSeg {
+public:
+	int flags;
+	GXSeg(uint s=0, uint e=0, int f=0):GSeg(s,e),flags(f) { }
+};
+
 //Data structure holding a query locus data (overlapping mRNAs on the same strand)
 // and also the accuracy data of all mRNAs of a query locus
 // (against all reference loci overlapping the same region)
@@ -357,7 +378,7 @@ public:
     GffObj* mrna_maxcov;  //transcript with maximum coverage (for main "ref" transcript)
     GffObj* mrna_maxscore; //transcript with maximum gscore (for major isoform)
     GList<GffObj> mrnas; //list of transcripts (isoforms) for this locus
-	GArray<GSeg> uexons; //list of unique exons (covered segments) in this region
+	GArray<GXSeg> uexons; //list of unique exons (covered segments) in this region
 	GArray<GSeg> mexons; //list of merged exons in this region
 	GIArray introns;
 	GList<GLocus> cmpovl; //temp list of overlapping qry/ref loci to compare to (while forming superloci)
@@ -371,7 +392,7 @@ public:
 	int spl_rare; // number of GC-AG, AT-AC and other rare splice site consensi
 	int spl_wrong; //number of "wrong" (unrecognized) splice site consensi
 	int ichains; //number of multi-exon mrnas
-	int ichainTP;
+	int ichainTP; //number of intron chains fully matching reference introns
 	int ichainATP;
 	int mrnaTP;
 	int mrnaATP;
@@ -396,7 +417,11 @@ public:
 			for (int i=0;i<mrna->exons.Count();i++) {
 				seg.start=mrna->exons[i]->start;
 				seg.end=mrna->exons[i]->end;
-				uexons.Add(seg);
+				int xterm=0;
+				if (i==0) xterm|=1;
+				if (i==mrna->exons.Count()-1) xterm|=2;
+				GXSeg xseg(seg.start, seg.end, xterm);
+				uexons.Add(xseg);
 				mexons.Add(seg);
 				if (i>0) {
 					seg.start=mrna->exons[i-1]->end+1;
@@ -555,7 +580,11 @@ public:
 				seg.start=mrna->exons[i]->start;
 				seg.end=mrna->exons[i]->end;
 				if (!ovlexons.Exists(i)) mexons.Add(seg);
-				uexons.Add(seg);
+				int xterm=0;
+				if (i==0) xterm|=1;
+				if (i==mrna->exons.Count()-1) xterm|=2;
+				GXSeg xseg(seg.start, seg.end, xterm);
+				uexons.Add(xseg);
 				GISeg iseg;
 				if (i>0) {
 					iseg.start=mrna->exons[i-1]->end+1;
@@ -595,12 +624,12 @@ public:
     GList<GLocus> rloci;
     GList<GffObj> qmrnas; //list of transcripts (isoforms) for this locus
     GArray<GSeg> qmexons; //list of merged exons in this region
-    GArray<GSeg> quexons; //list of unique exons (covered segments) in this region
+    GArray<GXSeg> quexons; //list of unique exons (covered segments) in this region
     GIArray qintrons; //list of unique exons (covered segments) in this region
     //same lists for reference:
     GList<GffObj> rmrnas; //list of transcripts (isoforms) for this locus
     GArray<GSeg> rmexons; //list of merged exons in this region
-    GArray<GSeg> ruexons; //list of unique exons (covered segments) in this region
+    GArray<GXSeg> ruexons; //list of unique exons (covered segments) in this region
     GArray<GISeg> rintrons; //list of unique exons (covered segments) in this region
     // store problematic introns for printing:
     GIArray i_missed; //missed reference introns (not overlapped by any qry intron)
@@ -635,7 +664,7 @@ public:
 	
     //--- accuracy data after compared to ref loci:
   int locusQTP;
-	int locusTP;
+  int locusTP; // +1 if ichainTP+mrnaTP > 0
   int locusAQTP;
 	int locusATP; // 1 if ichainATP + mrnaATP > 0
 	int locusFP;
@@ -650,13 +679,13 @@ public:
 	int mrnaAFN;
 	int mrnaAFP;
 	//---intron level accuracy (comparing the ordered set of splice sites):
-	int ichainTP; // number of qry intron chains covering a reference intron chain
-	// (covering meaning that the ordered set of reference splice sites
-	//  is the same with a ordered subset of the query splice sites)
-	int ichainFP; // number of qry intron chains not covering a reference intron chain
+	int ichainTP; // number of qry intron chains fully matching a reference intron chain
+	
+	int ichainFP; // number of qry intron chains not matching a reference intron chain
 	int ichainFN; // number of ref intron chains in this region not being covered by a reference intron chain
-	// same as above, but approximate -- allowing a 10bp distance error for splice sites
-	int ichainATP;
+	// same as above, but Approximate -- allowing a 5bp distance around splice site coordinates
+	int ichainATP; //as opposed to ichainTP, this also includes ref intron chains which are 
+                   //sub-chains of qry intron chains (rare cases)
 	int ichainAFP;
 	int ichainAFN;
 	//---projected features ---
@@ -804,9 +833,9 @@ public:
 		mrnaTP+=s.mrnaTP;
 		mrnaATP+=s.mrnaATP;
 		locusTP+=s.locusTP;
-    locusQTP+=s.locusQTP;
+		locusQTP+=s.locusQTP;
 		locusATP+=s.locusATP;
-    locusAQTP+=s.locusAQTP;
+		locusAQTP+=s.locusAQTP;
 		m_exons+=s.m_exons;
 		w_exons+=s.w_exons;
 		m_introns+=s.m_introns;
@@ -975,7 +1004,7 @@ class GTrackLocus:public GSeg {
    void addQCl(int q, GQCluster* qcl, GLocus* lnkloc) {
       for (int i=0;i<qcl->qloci.Count();i++) {
          GLocus* loc=qcl->qloci[i];
-         if (loc==lnkloc || loc->t_ptr==this) continue;
+         if (loc==lnkloc) continue; // or if loc->t_ptr==this ?
          hasQloci=true;
          loc->t_ptr=this;
          qcls[q]->addLocus(loc);
